PYTHON_BASEDIR='/opt/vdist-python'
PYTHON_VERSION='2.7.9'
LOCAL_PROFILES_DIR='buildprofiles'
LOCAL_PROFILES_FILE='profiles.json'
BUILD_BASEDIR='dist'
SCRATCH_BUILDSCRIPT_NAME='buildscript.sh'
SCRATCH_DIR='scratch'
SHARED_DIR='/work'
PACKAGE_BUILD_ROOT='/opt'
